@@ -1,14 +1,22 @@
 
+import ast
+import configparser
+import datetime
+import json
+import os
+import re
+import shutil
+import subprocess
+import sys
+import time
+from pathlib import Path
+
 import git
-import re, subprocess
 from git import RemoteProgress
-from pathlib import Path
-import os, sys, ast, shutil
-import configparser
-import json
-import datetime, time
-from pymvngit.utils import Utils, Emoticons, Colors
+
 from pymvngit.console import Console
+from pymvngit.utils import Colors, Emoticons, Utils
+
 
 class CloneProgress(RemoteProgress):
     def update(self, op_code, cur_count, max_count=None, msg=''):
@@ -57,6 +65,8 @@ class PyGit:
         # custom command    
         elif "listTags".upper() == self._command.upper() or "listTag".upper() == self._command.upper():
             self._listTags()     
+        elif "$git" == self._command:
+            self._executeCommandArbitrarily()
         else:
             Console.error("Git Command {} does not exist!".format(Colors.BIRED +  self._command + Colors.RESET))
             self._executed = False
@@ -80,7 +90,7 @@ class PyGit:
                added = True
                origin = repo.remotes.origin
                
-               msg  = Colors.BLUE + "Added " + Colors.GREEN + "repository {}".format(Colors.IBLUE +p["name"] + Colors.GREEN) 
+               msg  = Colors.BLUE + "Added " + Colors.GREEN + "project {}".format(Colors.IBLUE +p["name"] + Colors.GREEN) 
                isOK = True
                try:
                    if addAll:
@@ -197,7 +207,7 @@ class PyGit:
                commited = True
                origin = repo.remotes.origin
                
-               msg  = Colors.BLUE + "Commited " + Colors.GREEN + "repository {}".format(Colors.IBLUE +p["name"] + Colors.GREEN)
+               msg  = Colors.BLUE + "Commited " + Colors.GREEN + "project {}".format(Colors.IBLUE +p["name"] + Colors.GREEN)
                isOK = True
                try:
                    repo.git.commit(m=messageCommit)
@@ -218,17 +228,75 @@ class PyGit:
                Console.printLine(Console.addOkMsg(LABEL_LENGTH,m))
                del m    
 
+    def _executeCommandArbitrarily(self):
+        # Execute a Git Command arbitraryly against a Git Repository, exactly the way the user asked (no input or output interpretations)
+        # She/He must use this at her/his own risk!
+
+        # path = "/home/ualter/Developer/teachstore/teachstore-course"
+        # Directly
+        # result = git.Git(path).execute(["git", "status"])
+        # OR
+        # Referenced
+        # repo   = git.Repo("path")
+        #result = repo.git.execute(["git", "status"])
+        #print(result)
+
+        msg = ""
+        ok  = "Done!"  + Emoticons.ok()
+
+        lenLabelProject = 0
+        for p in self._projects:
+            if lenLabelProject < len(p["name"]):
+               lenLabelProject = len(p["name"])
+        lenLabelProject += 4
+
+        if isinstance(self._arguments, list):
+           if len(self._arguments) > 1:
+              arbitraryGitCommand = self._arguments
+           else: 
+              arbitraryGitCommand = self._arguments[0].split(" ") 
+        arbitraryGitCommand.insert(0,"git")
+
+        #clean spaces
+        arbitraryGitCommandReady = []
+        for a in arbitraryGitCommand:
+            arbitraryGitCommandReady.append(a.replace(" ",""))
+
+        for p in self._projects:
+            repo  = git.Repo(p["path"])
+
+            activeBranch    = str(repo.active_branch.name)
+            projectName     = p["name"] + " "
+            msg = Colors.BLUE + "Executing Git Command over " + Colors.GREEN + "project {}[{}{}]".format(Colors.IBLUE + projectName.ljust(lenLabelProject-1,"-")+ ">" + Colors.GREEN, Emoticons.blocks() ,Colors.IGREEN + activeBranch + Colors.GREEN)
+            msg = Console.addToMsg(LABEL_LENGTH,msg,ok)
+
+            result = repo.git.execute(arbitraryGitCommandReady)
+            result = result.replace("\n","\n       ")
+            msg += "\n       " + Colors.IBLACK +  result + Colors.GREEN + "\n"
+
+            Console.printLine(msg)
+
     def _status(self):
         msg = ""
         magnifier = "<< "  + Emoticons.magnifier()
         ok        = "Ok!"  + Emoticons.ok()
+
+        lenLabelProject = 0
+        for p in self._projects:
+            if lenLabelProject < len(p["name"]):
+               lenLabelProject = len(p["name"])
+        lenLabelProject += 4       
+
         for p in self._projects:
-            nothingToCommit = True
-            msg = Colors.BLUE + "Checking Status " + Colors.GREEN + "repository {}".format(Colors.IBLUE +p["name"] + Colors.GREEN)
-            msg = Console.addToMsg(LABEL_LENGTH,msg,magnifier)
 
             repo  = git.Repo(p["path"])
             items = repo.index.diff(None)
+
+            nothingToCommit = True
+            activeBranch    = str(repo.active_branch.name)
+            projectName     = p["name"] + " "
+            msg = Colors.BLUE + "Checking Status " + Colors.GREEN + "project {}[{}{}]".format(Colors.IBLUE + projectName.ljust(lenLabelProject-1,"-")+ ">" + Colors.GREEN, Emoticons.blocks() ,Colors.IGREEN + activeBranch + Colors.GREEN)
+            msg = Console.addToMsg(LABEL_LENGTH,msg,magnifier)
             
             if len(items) > 0:
                 nothingToCommit = False
