@@ -11,22 +11,19 @@ ualter.junior@gmail.com
 
 import os, sys, configparser, logging, re
 import xml.etree.ElementTree as ET
+from os.path import expanduser
 from pathlib import Path
 from urllib.request import pathname2url
 from tinydb import TinyDB, Query, where
 from tinydb.storages import JSONStorage
-from tinydb.middlewares import CachingMiddleware
-from os.path import expanduser
+from pymvngit import __version__ as VERSION
+from pymvngit import __author__ as AUTHOR
 from pymvngit.pygit import PyGit
 from pymvngit.console import Console
 from pymvngit.utils import Utils, Colors, Emoticons
 from pymvngit.prettytable import PrettyTable
 from pymvngit.tableargs import TableArgs
 
-__author__    = "Ualter Otoni Pereira"
-__copyright__ = "Ualter Otoni Pereira"
-__license__   = "MIT"
-
 _logger = logging.getLogger("app." + __name__)
 _logger.setLevel(logging.INFO)
 
@@ -57,21 +54,19 @@ class PyMvnGit:
 
     def _init(self):
         if not os.path.exists(USER_DIR):
-           os.makedirs(USER_DIR)
+            os.makedirs(USER_DIR)
         self._initLog()
         self._initIniFile()
 
     def _initLog(self):
         if not os.path.exists(LOG_DIR):
-           os.makedirs(LOG_DIR)
-
-        rootLogger = logging.getLogger()
-        logFormatter = logging.Formatter("%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s]  %(message)s")   
+            os.makedirs(LOG_DIR)
+        logging.Formatter("%(asctime)s [%(threadName)-12.12s][%(levelname)-5.5s]  %(message)s")
 
     def _initDBMacroDefault(self):
         createDefault = False
         if len(self._databaseMacro.all()) == 0:
-           createDefault = True
+            createDefault = True
 
         if createDefault:   
             self._databaseMacro.insert({
@@ -80,7 +75,7 @@ class PyMvnGit:
                 "executions":[
                     {
                         "tool":"git",
-                        "command":"status",
+                        "command":"st",
                         "projects":[]
                     }
                 ]
@@ -168,11 +163,56 @@ class PyMvnGit:
 
     def _connectDBs(self, startDatabases=False):
         if not os.path.exists(self._fileDBProject) and not startDatabases:
-           print("")
-           Console.error("{green}There's no repository created for {iblue}{curdir}{green}, please start it using: {iblue}pymvngit start{reset}". \
-                         format(curdir=self._fileDBProject,iblue=Colors.IBLUE,igreen=Colors.IGREEN,green=Colors.GREEN,reset=Colors.RESET))
-           sys.exit()              
-
+           # Check Environment Variable:
+           if 'PYMVNGIT_REPO' in os.environ and os.environ['PYMVNGIT_REPO'] != "":
+              defaultRepository = os.environ['PYMVNGIT_REPO']
+              foundRepo = False
+              for path in Path(USER_DIR).rglob('*.json'):
+               if path.name != NAME_DB_MACROS:
+                  nameRepo = path.name[path.name.rfind("_")+1:].replace(".json","").lower()
+                  if defaultRepository.lower().strip() == nameRepo.strip():
+                     self._fileDBProject = DB_PATH_PROJECT.replace("#",path.name.replace(".json","")) 
+                     foundRepo = True
+                     break
+              if not foundRepo:      
+                 Console.error(Colors.GREEN + "The environment variable value \"PYMVNGIT={}{}\" was not found as a created repository!".format(Colors.IBLUE + defaultRepository + Colors.RESET, Colors.GREEN))   
+                 sys.exit()    
+           else:
+              direc = self._decodeDirectoryName(self._fileDBProject)
+              Console.clear()
+              Console.error("{green}There's no repository created at the \n       current directory {iblue}{curdir}{green} \
+                            \n\n       Please start it using: {iblue}pymvngit {iyellow}start{green}  \
+                            \n\n       {magenta}OR -->{reset}  \
+                            \n\n       {green}Set a repository using an environment variable: \
+                            \n           {iyellow}export{green} PYMVNGIT_REPO{reset}={iblue}teachstore{green}  \
+                            \n\n       {magenta}OR -->{reset} \
+                            \n\n       {green}Choose a {iyellow}number{green} below:". \
+                            format(curdir=direc,iblue=Colors.IBLUE,green=Colors.GREEN,blue=Colors.BLUE,\
+                                   reset=Colors.RESET,magenta=Colors.IMAGENTA,iyellow=Colors.IYELLOW), False)
+              #os.environ['HOME']  
+              index = 0
+              options = []
+              print(Colors.RESET + "       +----------------------------------+")
+              for path in Path(USER_DIR).rglob('*.json'):
+               if path.name != NAME_DB_MACROS:
+                  options.append(path.name)
+                  nameDecoded = self._decodeDirectoryName(path.name)
+                  nameRepo    = nameDecoded
+                  if nameDecoded.rfind("/") > 1:
+                     nameRepo    = nameDecoded[nameDecoded.rfind("/")+1:]
+                  index += 1
+                  line = "       {green}[{iblue}{idx:02d}{green}]{reset} - {green}{repo}{reset}" \
+                         .format(idx=index,repo=nameRepo,green=Colors.GREEN,iblue=Colors.IBLUE,reset=Colors.RESET)
+                  print(line)
+              print(Colors.RESET + "       +----------------------------------+")    
+              read = input(Colors.IGREEN + "       Number...: ")
+              print("")
+              if read != "":
+                 n = options[int(read)-1].replace(".json","")
+                 self._fileDBProject = DB_PATH_PROJECT.replace("#",n)
+              else:   
+                 sys.exit()              
+   
         self._databaseProject = TinyDB(self._fileDBProject, indent=4)
         self._databaseMacro = TinyDB(self._fileDBMacro, indent=4)
 
@@ -185,13 +225,14 @@ class PyMvnGit:
     def _currentDir(self):
         currentDirectory = os.getcwd()
         #For testing/development
-        #currentDirectory  = "/home/ualter/Developer/teachstore"
-        #currentDirectory = "d:\\Developer\\code\\teachstore"
-        #currentDirectory  = "/home/ualter/Developer"
-        return currentDirectory
+        return "/home/ualter/Developer/teachstore"
+        #return currentDirectory
 
     def _encodeDirectoryName(self, directory):
         return directory.replace("\\","_").replace("/","_").replace(":","_drive_").lower()
+    def _decodeDirectoryName(self, directory):
+        return directory.replace("_","/").replace("_","/").replace("//","/").replace("_drive_",":") \
+                        .replace(USER_DIR,"").replace(".json","").lower()    
 
     def listMacros(self, summary=False, jsonFormat=False):
         """
@@ -222,7 +263,7 @@ class PyMvnGit:
                   for execution in r["executions"]:
                       i += 1
                       tools += "{high}#{i}{co}[{cn}{tool}{co}:{cn}\"{carg}{cmd}{cn}\"{co}]{cn} " \
-                               .format(i=i,tool=execution["tool"],cmd=execution["command"], \
+                               .format(i=i,tool=execution["tool"],cmd=execution["command"].replace("$git",Colors.IRED+"$git"+Colors.CYAN), \
                                co=Colors.IBLUE,cn=Colors.GREEN,high=Colors.IGREEN,carg=Colors.CYAN)      
                       projects=", ".join(execution["projects"])
                       if projects == "": projects = "ALL" 
@@ -231,7 +272,7 @@ class PyMvnGit:
                   prettyTable.addRow(columns)
                   idx_lin    += 1
    
-              if idx_line > 1:
+              if idx_lin > 1:
                  prettyTable.sortByColumn(0)   
                  prettyTable.numberSeparator = True
                  prettyTable.ascendingOrder(True)
@@ -296,8 +337,14 @@ class PyMvnGit:
                   prettyTable.sortByColumn(0)   
                   prettyTable.numberSeparator = True
                   prettyTable.ascendingOrder(True)
-                  nameDecoded = path.name.replace("_","/").replace("//","/").replace("/drive",":")
-                  Console.printTable("FOLDER/REPOSITORY: {}{}{}".format(Colors.IGREEN,nameDecoded,Colors.GREEN), prettyTable, tableArgs)    
+                  nameDecoded = self._decodeDirectoryName(path.name)
+                  nameRepo    = nameDecoded
+                  if nameDecoded.rfind("/") > 1:
+                     nameRepo    = nameDecoded[nameDecoded.rfind("/")+1:]
+                  Console.printTable("NAME:{}{}{} - FOLDER:{}{}{}" \
+                         .format(Colors.IGREEN,nameRepo,Colors.RESET, \
+                                 Colors.IGREEN,nameDecoded,Colors.GREEN 
+                                ), prettyTable, tableArgs)    
                db.close()   
 
     def clone(self, repositories, localPath, branchName,isRepoListFile):
@@ -333,7 +380,7 @@ class PyMvnGit:
         isMaven = os.path.exists( os.path.join(currentDirectory,"pom.xml") )
         isGit   = os.path.exists( os.path.join(currentDirectory,".git") )
         if isMaven or isGit:
-           self._createModule(currentDirectory[currentDirectory.rfind("/")+1:],currentDirectory) 
+           self._createModule(currentDirectory[currentDirectory.rfind("/")+1:],currentDirectory,isMaven,isGit) 
 
         # Save each first-level subfolder as part of the repository
         for item in os.listdir(currentDirectory):
@@ -358,7 +405,7 @@ class PyMvnGit:
         self._databaseProject.insert(record)
         #Utils.printJSON(Utils.dictToJSON(record))
 
-    def executeMacro(self, macro):
+    def executeMacro(self, macro, repository=None):
         """
         \033[0;32mExecute a macro\033[0;0m
 
@@ -390,6 +437,11 @@ class PyMvnGit:
                cmd     = cmdArgs["command"]
                args    = cmdArgs["arguments"]
 
+               cmdView = cmd + " "
+
+               if cmdView == "$git ":
+                  cmdView = "(" + Colors.IBLACK + "arbitrary" + Colors.IBLUE + ") "
+
                printArgs = "" 
                for i in args:
                    if " " in i.strip(): 
@@ -398,7 +450,7 @@ class PyMvnGit:
                       printArgs += i 
 
                Console.title("{c2}COMMAND..........: {c1}{cur}{c2} of {c1}{tot}{c2} --> {cmd}"\
-                            .format(cur=index,tot=tot,cmd=Colors.ICYAN + ex["tool"] + Colors.CYAN + " " + cmd + " " + printArgs,\
+                            .format(cur=index,tot=tot,cmd=Colors.ICYAN + ex["tool"] + Colors.CYAN + " " + cmdView + printArgs,\
                             c1=Colors.IGREEN,cn=Colors.RESET,c2=Colors.CYAN),True,False)                          
                print("")             
 
@@ -447,7 +499,7 @@ class PyMvnGit:
            return result
 
         cmd  = command[:command.find(" ")].strip()
-        args = command[command.find(" "):].strip()
+        args = command[command.find(" "):].replace("$git","").strip()
         args = self._replaceArgsPlaceHolder(tool, command, args)
 
         result["command"]   = cmd
@@ -455,6 +507,18 @@ class PyMvnGit:
 
         return result
 
+    def executeGitCommand(self, command, args):
+        if not self._connectedDatabase:
+           self._connectDBs()
+
+        projects = self._databaseProject.all()
+        #for p in projects:
+        #    print(p["name"])
+
+        del args[0:2]
+        pygit = PyGit("$"+command,args,projects) 
+        pygit.execute()
+
     def _replaceArgsPlaceHolder(self, tool, originalCmd, args):
         userArguments = {}
         for idx, val in enumerate(sys.argv):
@@ -474,8 +538,12 @@ class PyMvnGit:
 
            if isTherePlaceholders:
               if regexp.search(args):
+                 if "$git" in originalCmd:
+                    originalCmd = originalCmd.replace("$git","")
+                 else:   
+                    originalCmd = " " + originalCmd
                  print("")
-                 print("  " + Emoticons.error() + " " + Colors.GREEN + "The \033[1;91m{} {}\033[32m contains arguments not passed!".format(tool, originalCmd))
+                 print("  " + Emoticons.error() + " " + Colors.GREEN + "The \033[1;91m{}{}\033[32m contains arguments not passed!".format(tool, originalCmd))
                  print("       Please, check it...")
                  print("")
                  Console.printFooter()
@@ -501,15 +569,17 @@ def main():
     command = sys.argv[1]   
     if "help".upper() == command.upper() or "-h".upper() == command.upper():
        syntax()
-    elif "listProjects".upper() == command.upper():
+    elif "listProjects".upper() == command.upper() or "lp".upper() == command.upper():
        pymvngit.listProjects()
     elif "editMacros".upper() == command.upper() or "editMacro".upper() == command.upper() or "emac".upper() == command.upper():
        pymvngit.editMacros()   
-    elif "listMacros".upper() == command.upper():
+    elif "listMacros".upper() == command.upper() or "lm".upper() == command.upper():
        summary = True
        if len(sys.argv) > 2 and sys.argv[2] == "-v":
           summary = False
        pymvngit.listMacros(summary)
+    elif "git" == command:
+       pymvngit.executeGitCommand(command, sys.argv)
     elif "start".upper() == command.upper():
        pymvngit.start()
     elif "clone".upper() == command.upper():
@@ -542,48 +612,69 @@ def main():
                  branchName = sys.argv[index].strip()   
 
           pymvngit.clone(repositories,localPath,branchName,isRepoListFile)       
-    else:   
+    else:
        pymvngit.executeMacro(command)
 
 def syntax(clearScreen=True):
     if clearScreen:
        Console.clear()
-    Console.printHeaderCommand("PyGitMvn Tool")    
-    print("""
-        {green}Tool to manage a group of Git/Maven projects.
+    Console.printHeaderCommand("PyGitMvn Tool {}v{}{}".format(Colors.IBLUE,VERSION,Colors.GREEN))    
+    print("""{green}
+        Tool to manage a collection of Git/Maven projects as a single one. 
+        
+        These collection of projects might be a group of Microservices that belongs to the same Application
+        that needs (sometimes) to the be handled altogether, i.e. create a Git Tag.
 
         {iblue}MACROS{green}
         ------
-        A sequence of Git/Maven commands to run sequentially over project(s) of a repository.
-
+        A sequence of Git/Maven commands to run sequentially over project(s) of a repository (JSON file format).
+        
         {iblue}REPOSITORY{green}
         ----------
-        A collection of Git/Maven projects that can be managed together.
+        A collection of Git/Maven projects that can be managed together (JSON file format).
 
         {iblue}FUNCTIONS:{green}
-        - start...........: {blue}Create a repository from the current directory. Every subfolder will be 
-                            analyzed and if it's a Git/Maven project, will be part of this created repository.{green}
-        - clone...........: {blue}Create a repository cloning all the URL repository(ies) passed as argument
-                            {green}#{cyan}arguments:
-                            - Repositories.......: {cyan}(1){green} -r "https://github/projectA, https://github/projectB"{green}
-                                                      {cyan}or{green}
-                                                   {cyan}(2){green} -f file-name.txt     {cyan}# file with the list of repositories
-                            - Local path.........: {green}-p "/home/my/projects"{cyan}
-                            - Branch (optional)..: {green}-b "develop"{cyan}             #"branch" is the default value
-        - {cyan}[{green}macro{cyan}]{green}.........: {blue}Name of a macro to be executed.{green}
-        - listProjects....: {blue}List the projects part of the repository associated with the current directory.{green}
-        - listMacros......: {blue}List the macros configured. 
-                            {green}#{cyan}arguments:
-                            - Show more details, print a table for each one of them..: {green}{{-v}}{cyan}
-        {iblue}SYNTAX:{green}
-          pymvngit {iblue}[MACRO] {cyan}{{arguments}}
-        {iblue}EXAMPLES:{green}
-          pymvngit {iblue}build-all{green}
-          pymvngit {iblue}add-push{cyan} -A Patch v0.5.7 BugFixCrashing{green}
-          pymvngit {iblue}status-all{green}
-          pymvngit {iblue}install-commons{green}
-          pymvngit {iblue}ltags{green}
+        All functions will be launched against the current repository, based on the current directory
+        In case the current directory isn't a repository, the tool will  present a menu with  options
+        to choose among the repositories already created previously.
 
+        Another option is to set an environment variable with the name PYMVNGIT_REPO
+
+        - start...............: {blue}Create a repository from the current directory. Every subfolder will be 
+                                analyzed and if it's a Git/Maven project, will be part of this created repository.{green}
+                                Examples: 
+                                  pymvngit{cyan} start{green}
+
+        - clone...............: {blue}Create a repository cloning all the URL repository(ies) passed as argument
+                                {green}#{cyan}arguments:
+                                - Repositories.......: {cyan}(1){green} -r "https://github/projectA, https://github/projectB"{green}
+                                                          {cyan}or{green}
+                                                       {cyan}(2){green} -f file-name.txt     {cyan}# file with the list of repositories
+                                - Local path.........: {green}-p "/home/my/projects"{cyan}
+                                - Branch (optional)..: {green}-b "develop"{cyan}             # "master" is the default value{green}
+                                Examples: 
+                                  pymvngit{cyan} -r "https://github/projectA, https://github/projectB"{green}
+                                  pymvngit{cyan} -f my-projects.txt{green}
+
+        - git.................: {blue}Arbitrarily execute a Git command over a repository{cyan}
+                                git [command] {{arguments}}{green}
+                                Examples: 
+                                  pymvngit{cyan} git tag -n{green}
+                                  pymvngit{cyan} git remote show origin{green}
+         
+        - {cyan}[{green}macro{cyan}]{green}.............: {blue}Name of a macro to be executed (see below).{green}
+                                {iblue}SYNTAX:{green}pymvngit {iblue}[MACRO] {cyan}{{arguments}}{green}
+                                Examples:
+                                  pymvngit {cyan}build-all{green}
+                                  pymvngit {cyan}add-push{cyan} -A Patch v0.5.7 BugFixCrashing{green}
+                                  pymvngit {cyan}status-all{green}
+                                  pymvngit {cyan}install-commons{green}
+                                  pymvngit {cyan}ltags{green}
+
+        - listProjects (lp)...: {blue}List the projects part of the repository associated with the current directory.{green}
+        - listMacros (lm).....: {blue}List the macros configured. 
+                                {green}#{cyan}arguments:
+                                - Show more details, print a table for each one of them..: {green}{{-v}}{cyan}
     """.format(igreen=Colors.IGREEN,iblue=Colors.IBLUE,ired=Colors.IRED,blue=Colors.BLUE, \
                green=Colors.GREEN,reset=Colors.RESET,cyan=Colors.CYAN,icyan=Colors.ICYAN))
 
